# last modified 2009-11-11 by J. Fox

av.plot <- function (...) {
	.Deprecated("avPlot", package="car")
	avPlot(...)
}

av.plots <- function (...) {
	.Deprecated("avPlots", package="car")
	avPlots(...)
}

box.cox <- function (...) {
	.Deprecated("bcPower", package="car")
	bcPower(...)
}

bc <- function (...) {
	.Deprecated("bcPower", package="car")
	bcPower(...)
}

box.cox.powers <- function (...) {
	.Deprecated("powerTransform", package="car")
	powerTransform(...)
}

box.cox.var <- function (...) {
	.Deprecated("boxCoxVariable", package="car")
	boxCoxVariable(...)
}

box.tidwell <- function (...) {
	.Deprecated("boxTidwell", package="car")
	boxTidwell(...)
}

ceres.plot <- function (...) {
	.Deprecated("ceresPlot", package="car")
	ceresPlot(...)
}

ceres.plots <- function (...) {
	.Deprecated("ceresPlots", package="car")
	ceresPlots(...)
}

confidence.ellipse <- function (...) {
	.Deprecated("confidenceEllipse", package="car")
	confidenceEllipse(...)
}

cookd <- function (...) {
	.Deprecated("cooks.distance", package="stats")
	cooks.distance(...)
}

cr.plot <- function (...) {
	.Deprecated("crPlot", package="car")
	crPlot(...)
}

cr.plots <- function (...) {
	.Deprecated("crPlots", package="car")
	crPlots(...)
}

data.ellipse <- function (...) {
	.Deprecated("dataEllipse", package="car")
	dataEllipse(...)
}

durbin.watson <- function (...) {
	.Deprecated("durbinWatsonTest", package="car")
	durbinWatsonTest(...)
}

levene.test <- function (...) {
	.Deprecated("leveneTest", package="car")
	leveneTest(...)
}

leverage.plot <- function (...) {
	.Deprecated("leveragePlot", package="car")
	leveragePlot(...)
}

leverage.plots <- function (...) {
	.Deprecated("leveragePlots", package="car")
	leveragePlots(...)
}

linear.hypothesis <- function (...) {
	.Deprecated("linearHypothesis", package="car")
	linearHypothesis(...)
}

outlier.test <- function (...) {
	.Deprecated("outlierTest", package="car")
	outlierTest(...)
}

ncv.test <- function (...) {
	.Deprecated("ncvTest", package="car")
	ncvTest(...)
}

qq.plot <- function (...) {
	.Deprecated("qqPlot", package="car")
	qqPlot(...)
}

scatterplot.matrix <- function (...) {
	.Deprecated("scatterplotMatrix", package="car")
	scatterplotMatrix(...)
}

spread.level.plot <- function (...) {
	.Deprecated("spreadLevelPlot", package="car")
	spreadLevelPlot(...)
}